/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.languages.javascript.jsdi;

import java.util.List;
import java.util.Map;

/**
 * Describes a JavaScript script object. A script object has location information for all of the lines and functions contained within it.
 * 
 * @since 0.9
 */
public interface ScriptReference extends Mirror {

	/**
	 * Returns the complete live list of line locations in this script. <br>
	 * <br>
	 * This method can return <code>null</code>.
	 * 
	 * @return the line locations in this script
	 * @see Location
	 * @see #lineLocation(int)
	 */
	public List/* <Location> */allLineLocations();

	/**
	 * Returns the {@link Location} information for the given line number in this script.<br>
	 * <br>
	 * This method can return <code>null</code>.
	 * 
	 * @param lineNumber
	 * @return the {@link Location} information for the given line number.
	 * @see Location
	 * @see #allLineLocations()
	 */
	public Location lineLocation(int lineNumber);

	/**
	 * Returns the complete live list of function locations in this script.<br>
	 * <br>
	 * This method can return <code>null</code>.
	 * 
	 * @return the function locations in this script
	 * @see Location
	 * @see #functionLocation(String)
	 */
	public List/* <Location> */allFunctionLocations();

	/**
	 * Returns the {@link Location} information for a function with the given name in this script.<br>
	 * <br>
	 * This method can return <code>null</code>.
	 * 
	 * @param functionName
	 * @return the {@link Location} information for a function with the given name
	 * @see Location
	 * @see #allFunctionLocations()
	 */
	public Location functionLocation(String functionName);

	/**
	 * Returns the entire known source for this script at the time this method is called.<br>
	 * <br>
	 * This method can return <code>null</code>.
	 * 
	 * @return the source for this script
	 */
	public String source();

	/**
	 * Returns the path to the source of this script.<br>
	 * <br>
	 * This method can return <code>null</code>.
	 * 
	 * @return the path to the source of this script
	 */
	public String sourcePath();

	/**
	 * Returns a mapping of additional properties describing the source for this script.<br>
	 * <br>
	 * This method can return <code>null</code>.
	 * 
	 * @return a mapping of additional source properties for this script
	 */
	public Map sourceProperties();
}
