/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.languages.javascript.jsdi;

import java.util.List;

/**
 * Abstract description of a stack frame with-respect-to JavaScript debugging
 * 
 * @since 1.0
 */
public interface StackFrame extends Mirror {

	/**
	 * Returns the value of 'this' for the current frame.
	 * 
	 * @return the 'this' ObjectReference
	 * 
	 */
	public Variable thisObject();

	/**
	 * The visible variables for this StackFrame
	 * 
	 * @return the list of Variable objects for this StackFrame
	 */
	public List variables();

	/**
	 * The underlying value for this variable
	 * 
	 * @return the underlying value for the Variable object
	 */
	public Value getValue(Variable variable);

	/**
	 * The location associated with this stack frame
	 * 
	 * @return the current Location associated with this stack frame
	 */
	public Location location();

	/**
	 * Evaluates the given expression in the context of this stack frame
	 * 
	 * @return the result of evaluating the expression
	 */
	public Value evaluate(String expression);
}
