/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.languages.javascript.jsdi.rhino;

import java.util.Map;

import org.eclipse.e4.languages.javascript.debug.connect.JSONConstants;
import org.eclipse.e4.languages.javascript.jsdi.FunctionReference;
import org.eclipse.e4.languages.javascript.jsdi.ObjectReference;

/**
 * Rhino implementation of {@link ObjectReference}
 * 
 * @since 1.0
 */
public class ObjectReferenceImpl extends PropertiesReferenceImpl implements
		ObjectReference {

	private final String className;
	private final Number constructorRef;
	private final Number prototypeRef;

	/**
	 * Constructor
	 * 
	 * @param vm
	 * @param body
	 * @param frame
	 */
	public ObjectReferenceImpl(VirtualMachineImpl vm, Map body,
			StackFrameImpl frame) {
		super(vm, body, frame);
		this.className = (String) body.get(JSONConstants.CLASS_NAME);
		this.constructorRef = (Number) ((Map) body
				.get(JSONConstants.CONSTRUCTOR_FUNCTION))
				.get(JSONConstants.REF);
		this.prototypeRef = (Number) ((Map) body
				.get(JSONConstants.PROTOTYPE_OBJECT)).get(JSONConstants.REF);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.e4.languages.javascript.jsdi.ObjectReference#className()
	 */
	public String className() {
		return this.className;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.e4.languages.javascript.jsdi.ObjectReference#constructor()
	 */
	public FunctionReference constructor() {
		return (FunctionReference) frame.lookupValue(constructorRef);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.e4.languages.javascript.jsdi.ObjectReference#prototype()
	 */
	public ObjectReference prototype() {
		return (ObjectReference) frame.lookupValue(prototypeRef);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.e4.languages.javascript.jsdi.Value#getValueTypeName()
	 */
	public String getValueTypeName() {
		return JSONConstants.OBJECT;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Object"; //$NON-NLS-1$
	}
}
